@@ -3,8 +3,11 @@
 
 """Dataset Class for Real-World Logged Bandit Feedback."""
 from dataclasses import dataclass
+from logging import getLogger, basicConfig, INFO
 from pathlib import Path
-from typing import Optional, Tuple
+from typing import Optional
+from typing import Union
+from typing import Tuple
 
 import numpy as np
 import pandas as pd
@@ -15,6 +18,9 @@ from sklearn.utils import check_random_state
 from .base import BaseRealBanditDataset
 from ..types import BanditFeedback
 
+logger = getLogger(__name__)
+basicConfig(level=INFO)
+
 
 @dataclass
 class OpenBanditDataset(BaseRealBanditDataset):
@@ -33,8 +39,9 @@ class OpenBanditDataset(BaseRealBanditDataset):
     campaign: str
         One of the three possible campaigns considered in ZOZOTOWN, "all", "men", and "women".
 
-    data_path: Path, default=Path('./obd')
-        Path that stores Open Bandit Dataset.
+    data_path: Path, default=None
+        Path where the Open Bandit Dataset exists.
+        When `None` is given, this class downloads the example small-sized version of the dataset.
 
     dataset_name: str, default='obd'
         Name of the dataset.
@@ -42,28 +49,42 @@ class OpenBanditDataset(BaseRealBanditDataset):
     References
     ------------
     Yuta Saito, Shunsuke Aihara, Megumi Matsutani, Yusuke Narita.
-    "Large-scale Open Dataset, Pipeline, and Benchmark for Bandit Algorithms.", 2020.
+    "Open Bandit Dataset and Pipeline: Towards Realistic and Reproducible Off-Policy Evaluation.", 2020.
 
     """
 
     behavior_policy: str
     campaign: str
-    data_path: Path = Path("./obd")
+    data_path: Optional[Path] = None
     dataset_name: str = "obd"
 
     def __post_init__(self) -> None:
         """Initialize Open Bandit Dataset Class."""
-        assert self.behavior_policy in [
+        if self.behavior_policy not in [
             "bts",
             "random",
-        ], f"behavior_policy must be either of 'bts' or 'random', but {self.behavior_policy} is given"
-        assert self.campaign in [
+        ]:
+            raise ValueError(
+                f"behavior_policy must be either of 'bts' or 'random', but {self.behavior_policy} is given"
+            )
+
+        if self.campaign not in [
             "all",
             "men",
             "women",
-        ], f"campaign must be one of 'all', 'men', and 'women', but {self.campaign} is given"
-        assert isinstance(self.data_path, Path), f"data_path must be a Path type"
+        ]:
+            raise ValueError(
+                f"campaign must be one of 'all', 'men', and 'women', but {self.campaign} is given"
+            )
 
+        if self.data_path is None:
+            logger.info(
+                "When `data_path` is not given, this class downloads the example small-sized version of the Open Bandit Dataset."
+            )
+            self.data_path = Path(__file__).parent / "obd"
+        else:
+            if not isinstance(self.data_path, Path):
+                raise ValueError("data_path must be a Path type")
         self.data_path = self.data_path / self.behavior_policy / self.campaign
         self.raw_data_file = f"{self.campaign}.csv"
 
@@ -95,7 +116,7 @@ class OpenBanditDataset(BaseRealBanditDataset):
         cls,
         behavior_policy: str,
         campaign: str,
-        data_path: Path = Path("./obd"),
+        data_path: Optional[Path] = None,
         test_size: float = 0.3,
         is_timeseries_split: bool = False,
     ) -> float:
@@ -110,14 +131,15 @@ class OpenBanditDataset(BaseRealBanditDataset):
         campaign: str
             One of the three possible campaigns considered in ZOZOTOWN (i.e., "all", "men", and "women").
 
-        data_path: Path, default=Path('./obd')
-            Path that stores Open Bandit Dataset.
+        data_path: Path, default=None
+            Path where the Open Bandit Dataset exists.
+            When `None` is given, this class downloads the example small-sized version of the dataset.
 
         test_size: float, default=0.3
             If float, should be between 0.0 and 1.0 and represent the proportion of the dataset to include in the test split.
 
         is_timeseries_split: bool, default=False
-            If true, split the original logged badnit feedback data by time series.
+            If true, split the original logged bandit feedback data by time series.
 
         Returns
         ---------
@@ -127,13 +149,16 @@ class OpenBanditDataset(BaseRealBanditDataset):
             This parameter is used as a ground-truth policy value in the evaluation of OPE estimators.
 
         """
-        return (
-            cls(behavior_policy=behavior_policy, campaign=campaign, data_path=data_path)
-            .obtain_batch_bandit_feedback(
-                test_size=test_size, is_timeseries_split=is_timeseries_split
-            )["reward_test"]
-            .mean()
+        bandit_feedback = cls(
+            behavior_policy=behavior_policy, campaign=campaign, data_path=data_path
+        ).obtain_batch_bandit_feedback(
+            test_size=test_size, is_timeseries_split=is_timeseries_split
         )
+        if is_timeseries_split:
+            bandit_feedback_test = bandit_feedback[1]
+        else:
+            bandit_feedback_test = bandit_feedback
+        return bandit_feedback_test["reward"].mean()
 
     def load_raw_data(self) -> None:
         """Load raw open bandit dataset."""
@@ -154,7 +179,7 @@ class OpenBanditDataset(BaseRealBanditDataset):
 
         Note
         -----
-        This is the default feature engineering and please overide this method to
+        This is the default feature engineering and please override this method to
         implement your own preprocessing.
         see https://github.com/st-tech/zr-obp/blob/master/examples/examples_with_obd/custom_dataset.py for example.
 
@@ -171,7 +196,7 @@ class OpenBanditDataset(BaseRealBanditDataset):
 
     def obtain_batch_bandit_feedback(
         self, test_size: float = 0.3, is_timeseries_split: bool = False
-    ) -> BanditFeedback:
+    ) -> Union[BanditFeedback, Tuple[BanditFeedback, BanditFeedback]]:
         """Obtain batch logged bandit feedback.
 
         Parameters
@@ -179,36 +204,53 @@ class OpenBanditDataset(BaseRealBanditDataset):
         test_size: float, default=0.3
             If float, should be between 0.0 and 1.0 and represent the proportion of
             the dataset to include in the evaluation split.
+            This argument matters only when `is_timeseries_split=True` (the out-sample case).
 
         is_timeseries_split: bool, default=False
-            If true, split the original logged badnit feedback data by time series.
+            If true, split the original logged bandit feedback data by time series.
 
         Returns
         --------
         bandit_feedback: BanditFeedback
-            Batch logged bandit feedback collected by a behavior policy.
+            A dictionary containing batch logged bandit feedback data collected by a behavior policy.
+            The keys of the dictionary are as follows.
+            - n_rounds: number of rounds (size) of the logged bandit data
+            - n_actions: number of actions (:math:`|\mathcal{A}|`)
+            - action: action variables sampled by a behavior policy
+            - position: positions where actions are recommended
+            - reward: reward variables
+            - pscore: action choice probabilities by a behavior policy
+            - context: context vectors such as user-related features and user-item affinity scores
+            - action_context: item-related context vectors
 
         """
         if is_timeseries_split:
-            assert isinstance(test_size, float) & (
-                0 < test_size < 1
-            ), f"test_size must be a float in the (0,1) interval, but {test_size} is given"
+            if not isinstance(test_size, float) or (test_size <= 0 or test_size >= 1):
+                raise ValueError(
+                    f"test_size must be a float in the (0,1) interval, but {test_size} is given"
+                )
             n_rounds_train = np.int(self.n_rounds * (1.0 - test_size))
-            return dict(
+            bandit_feedback_train = dict(
                 n_rounds=n_rounds_train,
                 n_actions=self.n_actions,
                 action=self.action[:n_rounds_train],
-                action_test=self.action[n_rounds_train:],
                 position=self.position[:n_rounds_train],
-                position_test=self.position[n_rounds_train:],
                 reward=self.reward[:n_rounds_train],
-                reward_test=self.reward[n_rounds_train:],
                 pscore=self.pscore[:n_rounds_train],
-                pscore_test=self.pscore[n_rounds_train:],
                 context=self.context[:n_rounds_train],
-                context_test=self.context[n_rounds_train:],
                 action_context=self.action_context,
             )
+            bandit_feedback_test = dict(
+                n_rounds=(self.n_rounds - n_rounds_train),
+                n_actions=self.n_actions,
+                action=self.action[n_rounds_train:],
+                position=self.position[n_rounds_train:],
+                reward=self.reward[n_rounds_train:],
+                pscore=self.pscore[n_rounds_train:],
+                context=self.context[n_rounds_train:],
+                action_context=self.action_context,
+            )
+            return bandit_feedback_train, bandit_feedback_test
         else:
             return dict(
                 n_rounds=self.n_rounds,
@@ -216,7 +258,6 @@ class OpenBanditDataset(BaseRealBanditDataset):
                 action=self.action,
                 position=self.position,
                 reward=self.reward,
-                reward_test=self.reward,
                 pscore=self.pscore,
                 context=self.context,
                 action_context=self.action_context,
@@ -235,9 +276,10 @@ class OpenBanditDataset(BaseRealBanditDataset):
         test_size: float, default=0.3
             If float, should be between 0.0 and 1.0 and represent the proportion of
             the dataset to include in the evaluation split.
+            This argument matters only when `is_timeseries_split=True` (the out-sample case).
 
         is_timeseries_split: bool, default=False
-            If true, split the original logged badnit feedback data by time series.
+            If true, split the original logged bandit feedback data by time series.
 
         random_state: int, default=None
             Controls the random seed in bootstrap sampling.
@@ -245,12 +287,26 @@ class OpenBanditDataset(BaseRealBanditDataset):
         Returns
         --------
         bandit_feedback: BanditFeedback
-            Logged bandit feedback sampled independently from the original data with replacement.
+            A dictionary containing logged bandit feedback data sampled independently from the original data with replacement.
+            The keys of the dictionary are as follows.
+            - n_rounds: number of rounds (size) of the logged bandit data
+            - n_actions: number of actions
+            - action: action variables sampled by a behavior policy
+            - position: positions where actions are recommended by a behavior policy
+            - reward: reward variables
+            - pscore: action choice probabilities by a behavior policy
+            - context: context vectors such as user-related features and user-item affinity scores
+            - action_context: item-related context vectors
 
         """
-        bandit_feedback = self.obtain_batch_bandit_feedback(
-            test_size=test_size, is_timeseries_split=is_timeseries_split
-        )
+        if is_timeseries_split:
+            bandit_feedback = self.obtain_batch_bandit_feedback(
+                test_size=test_size, is_timeseries_split=is_timeseries_split
+            )[0]
+        else:
+            bandit_feedback = self.obtain_batch_bandit_feedback(
+                test_size=test_size, is_timeseries_split=is_timeseries_split
+            )
         n_rounds = bandit_feedback["n_rounds"]
         random_ = check_random_state(random_state)
         bootstrap_idx = random_.choice(np.arange(n_rounds), size=n_rounds, replace=True)
