@@ -8,7 +8,7 @@ from typing import Dict, Optional
 
 import numpy as np
 
-from ..utils import estimate_confidence_interval_by_bootstrap
+from ..utils import estimate_confidence_interval_by_bootstrap, check_ope_inputs
 
 
 @dataclass
@@ -67,8 +67,8 @@ class ReplayMethod(BaseOffPolicyEstimator):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         action_dist: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> np.ndarray:
         """Estimate rewards for each round.
@@ -81,7 +81,10 @@ class ReplayMethod(BaseOffPolicyEstimator):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
+        action_dist: array-like, shape (n_rounds, n_actions, len_list)
+            Action choice probabilities by the evaluation policy (must be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
+
+        position: array-like, shape (n_rounds,), default=None
             Positions of each round in the given logged bandit feedback.
 
         Returns
@@ -90,6 +93,8 @@ class ReplayMethod(BaseOffPolicyEstimator):
             Rewards estimated by the Replay Method for each round.
 
         """
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
         action_match = np.array(
             action_dist[np.arange(action.shape[0]), action, position] == 1
         )
@@ -102,8 +107,8 @@ class ReplayMethod(BaseOffPolicyEstimator):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         action_dist: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> float:
         """Estimate policy value of an evaluation policy.
@@ -116,7 +121,10 @@ class ReplayMethod(BaseOffPolicyEstimator):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
+        action_dist: array-like, shape (n_rounds, n_actions, len_list)
+            Action choice probabilities by the evaluation policy (must be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
+
+        position: array-like, shape (n_rounds,), default=None
             Positions of each round in the given logged bandit feedback.
 
         Returns
@@ -125,16 +133,30 @@ class ReplayMethod(BaseOffPolicyEstimator):
             Estimated policy value (performance) of a given evaluation policy.
 
         """
+        if not isinstance(reward, np.ndarray):
+            raise ValueError("reward must be ndarray")
+        if not isinstance(action, np.ndarray):
+            raise ValueError("action must be ndarray")
+
+        check_ope_inputs(
+            action_dist=action_dist, position=position, action=action, reward=reward
+        )
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
+
         return self._estimate_round_rewards(
-            reward=reward, action=action, position=position, action_dist=action_dist,
+            reward=reward,
+            action=action,
+            position=position,
+            action_dist=action_dist,
         ).mean()
 
     def estimate_interval(
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         action_dist: np.ndarray,
+        position: Optional[np.ndarray] = None,
         alpha: float = 0.05,
         n_bootstrap_samples: int = 100,
         random_state: Optional[int] = None,
@@ -150,7 +172,10 @@ class ReplayMethod(BaseOffPolicyEstimator):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
+        action_dist: array-like, shape (n_rounds, n_actions, len_list)
+            Action choice probabilities by the evaluation policy (must be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
+
+        position: array-like, shape (n_rounds,), default=None
             Positions of each round in the given logged bandit feedback.
 
         alpha: float, default=0.05
@@ -168,8 +193,22 @@ class ReplayMethod(BaseOffPolicyEstimator):
             Dictionary storing the estimated mean and upper-lower confidence bounds.
 
         """
+        if not isinstance(reward, np.ndarray):
+            raise ValueError("reward must be ndarray")
+        if not isinstance(action, np.ndarray):
+            raise ValueError("action must be ndarray")
+
+        check_ope_inputs(
+            action_dist=action_dist, position=position, action=action, reward=reward
+        )
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
+
         estimated_round_rewards = self._estimate_round_rewards(
-            reward=reward, action=action, position=position, action_dist=action_dist,
+            reward=reward,
+            action=action,
+            position=position,
+            action_dist=action_dist,
         )
         return estimate_confidence_interval_by_bootstrap(
             samples=estimated_round_rewards,
@@ -220,9 +259,9 @@ class InverseProbabilityWeighting(BaseOffPolicyEstimator):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> np.ndarray:
         """Estimate rewards for each round.
@@ -235,18 +274,23 @@ class InverseProbabilityWeighting(BaseOffPolicyEstimator):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
+        action_dist: array-like, shape (n_rounds, n_actions, len_list)
+            Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
+
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         estimated_rewards: array-like, shape (n_rounds,)
             Rewards estimated by IPW for each round.
 
         """
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
         iw = action_dist[np.arange(action.shape[0]), action, position] / pscore
         return reward * iw
 
@@ -254,9 +298,9 @@ class InverseProbabilityWeighting(BaseOffPolicyEstimator):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> np.ndarray:
         """Estimate policy value of an evaluation policy.
@@ -269,21 +313,38 @@ class InverseProbabilityWeighting(BaseOffPolicyEstimator):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
         action_dist: array-like, shape (n_rounds, n_actions, len_list)
             Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         V_hat: float
             Estimated policy value (performance) of a given evaluation policy.
 
         """
+        if not isinstance(reward, np.ndarray):
+            raise ValueError("reward must be ndarray")
+        if not isinstance(action, np.ndarray):
+            raise ValueError("action must be ndarray")
+        if not isinstance(pscore, np.ndarray):
+            raise ValueError("pscore must be ndarray")
+
+        check_ope_inputs(
+            action_dist=action_dist,
+            position=position,
+            action=action,
+            reward=reward,
+            pscore=pscore,
+        )
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
+
         return self._estimate_round_rewards(
             reward=reward,
             action=action,
@@ -296,9 +357,9 @@ class InverseProbabilityWeighting(BaseOffPolicyEstimator):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
+        position: Optional[np.ndarray] = None,
         alpha: float = 0.05,
         n_bootstrap_samples: int = 10000,
         random_state: Optional[int] = None,
@@ -314,9 +375,6 @@ class InverseProbabilityWeighting(BaseOffPolicyEstimator):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
@@ -324,6 +382,9 @@ class InverseProbabilityWeighting(BaseOffPolicyEstimator):
             Action choice probabilities
             by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         alpha: float, default=0.05
             P-value.
 
@@ -339,6 +400,23 @@ class InverseProbabilityWeighting(BaseOffPolicyEstimator):
             Dictionary storing the estimated mean and upper-lower confidence bounds.
 
         """
+        if not isinstance(reward, np.ndarray):
+            raise ValueError("reward must be ndarray")
+        if not isinstance(action, np.ndarray):
+            raise ValueError("action must be ndarray")
+        if not isinstance(pscore, np.ndarray):
+            raise ValueError("pscore must be ndarray")
+
+        check_ope_inputs(
+            action_dist=action_dist,
+            position=position,
+            action=action,
+            reward=reward,
+            pscore=pscore,
+        )
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
+
         estimated_round_rewards = self._estimate_round_rewards(
             reward=reward,
             action=action,
@@ -397,9 +475,9 @@ class SelfNormalizedInverseProbabilityWeighting(InverseProbabilityWeighting):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> np.ndarray:
         """Estimate rewards for each round.
@@ -412,21 +490,23 @@ class SelfNormalizedInverseProbabilityWeighting(InverseProbabilityWeighting):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
         action_dist: array-like, shape (n_rounds, n_actions, len_list)
             Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         estimated_rewards: array-like, shape (n_rounds,)
             Rewards estimated by the SNIPW estimator for each round.
 
         """
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
         iw = action_dist[np.arange(action.shape[0]), action, position] / pscore
         return reward * iw / iw.mean()
 
@@ -477,63 +557,80 @@ class DirectMethod(BaseOffPolicyEstimator):
 
     def _estimate_round_rewards(
         self,
-        position: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> float:
         """Estimate policy value of an evaluation policy.
 
         Parameters
         ----------
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         action_dist: array-like, shape (n_rounds, n_actions, len_list)
             Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
 
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         estimated_rewards: array-like, shape (n_rounds,)
             Rewards estimated by the DM estimator for each round.
 
         """
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
         n_rounds = position.shape[0]
         q_hat_at_position = estimated_rewards_by_reg_model[
             np.arange(n_rounds), :, position
         ]
         pi_e_at_position = action_dist[np.arange(n_rounds), :, position]
-        return np.average(q_hat_at_position, weights=pi_e_at_position, axis=1,)
+        return np.average(
+            q_hat_at_position,
+            weights=pi_e_at_position,
+            axis=1,
+        )
 
     def estimate_policy_value(
         self,
-        position: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> float:
         """Estimate policy value of an evaluation policy.
 
         Parameters
         ----------
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         action_dist: array-like, shape (n_rounds, n_actions, len_list)
             Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
 
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         V_hat: float
             Estimated policy value (performance) of a given evaluation policy.
 
         """
+        if not isinstance(estimated_rewards_by_reg_model, np.ndarray):
+            raise ValueError("estimated_rewards_by_reg_model must be ndarray")
+
+        check_ope_inputs(
+            action_dist=action_dist,
+            estimated_rewards_by_reg_model=estimated_rewards_by_reg_model,
+            position=position,
+        )
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
+
         return self._estimate_round_rewards(
             position=position,
             estimated_rewards_by_reg_model=estimated_rewards_by_reg_model,
@@ -542,9 +639,9 @@ class DirectMethod(BaseOffPolicyEstimator):
 
     def estimate_interval(
         self,
-        position: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
         alpha: float = 0.05,
         n_bootstrap_samples: int = 10000,
         random_state: Optional[int] = None,
@@ -554,15 +651,15 @@ class DirectMethod(BaseOffPolicyEstimator):
 
         Parameters
         ----------
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         action_dist: array-like, shape (n_rounds, n_actions, len_list)
             Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
 
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         alpha: float, default=0.05
             P-value.
 
@@ -578,6 +675,17 @@ class DirectMethod(BaseOffPolicyEstimator):
             Dictionary storing the estimated mean and upper-lower confidence bounds.
 
         """
+        if not isinstance(estimated_rewards_by_reg_model, np.ndarray):
+            raise ValueError("estimated_rewards_by_reg_model must be ndarray")
+
+        check_ope_inputs(
+            action_dist=action_dist,
+            estimated_rewards_by_reg_model=estimated_rewards_by_reg_model,
+            position=position,
+        )
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
+
         estimated_round_rewards = self._estimate_round_rewards(
             position=position,
             estimated_rewards_by_reg_model=estimated_rewards_by_reg_model,
@@ -592,7 +700,7 @@ class DirectMethod(BaseOffPolicyEstimator):
 
 
 @dataclass
-class DoublyRobust(InverseProbabilityWeighting):
+class DoublyRobust(BaseOffPolicyEstimator):
     """Estimate the policy value by Doubly Robust (DR).
 
     Note
@@ -643,10 +751,10 @@ class DoublyRobust(InverseProbabilityWeighting):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> np.ndarray:
         """Estimate rewards for each round.
@@ -659,9 +767,6 @@ class DoublyRobust(InverseProbabilityWeighting):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
@@ -671,12 +776,17 @@ class DoublyRobust(InverseProbabilityWeighting):
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         estimated_rewards: array-like, shape (n_rounds,)
             Rewards estimated by the DR estimator for each round.
 
         """
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
         n_rounds = action.shape[0]
         iw = action_dist[np.arange(n_rounds), action, position] / pscore
         q_hat_at_position = estimated_rewards_by_reg_model[
@@ -687,7 +797,9 @@ class DoublyRobust(InverseProbabilityWeighting):
         ]
         pi_e_at_position = action_dist[np.arange(n_rounds), :, position]
         estimated_rewards = np.average(
-            q_hat_at_position, weights=pi_e_at_position, axis=1,
+            q_hat_at_position,
+            weights=pi_e_at_position,
+            axis=1,
         )
         estimated_rewards += iw * (reward - q_hat_factual)
         return estimated_rewards
@@ -696,10 +808,10 @@ class DoublyRobust(InverseProbabilityWeighting):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
     ) -> float:
         """Estimate policy value of an evaluation policy.
 
@@ -711,9 +823,6 @@ class DoublyRobust(InverseProbabilityWeighting):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
@@ -723,12 +832,35 @@ class DoublyRobust(InverseProbabilityWeighting):
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         V_hat: float
             Estimated policy value by the DR estimator.
 
         """
+        if not isinstance(estimated_rewards_by_reg_model, np.ndarray):
+            raise ValueError("estimated_rewards_by_reg_model must be ndarray")
+        if not isinstance(reward, np.ndarray):
+            raise ValueError("reward must be ndarray")
+        if not isinstance(action, np.ndarray):
+            raise ValueError("action must be ndarray")
+        if not isinstance(pscore, np.ndarray):
+            raise ValueError("pscore must be ndarray")
+
+        check_ope_inputs(
+            action_dist=action_dist,
+            position=position,
+            action=action,
+            reward=reward,
+            pscore=pscore,
+            estimated_rewards_by_reg_model=estimated_rewards_by_reg_model,
+        )
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
+
         return self._estimate_round_rewards(
             reward=reward,
             action=action,
@@ -742,10 +874,10 @@ class DoublyRobust(InverseProbabilityWeighting):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
         alpha: float = 0.05,
         n_bootstrap_samples: int = 10000,
         random_state: Optional[int] = None,
@@ -761,9 +893,6 @@ class DoublyRobust(InverseProbabilityWeighting):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
@@ -773,6 +902,9 @@ class DoublyRobust(InverseProbabilityWeighting):
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         alpha: float, default=0.05
             P-value.
 
@@ -788,6 +920,26 @@ class DoublyRobust(InverseProbabilityWeighting):
             Dictionary storing the estimated mean and upper-lower confidence bounds.
 
         """
+        if not isinstance(estimated_rewards_by_reg_model, np.ndarray):
+            raise ValueError("estimated_rewards_by_reg_model must be ndarray")
+        if not isinstance(reward, np.ndarray):
+            raise ValueError("reward must be ndarray")
+        if not isinstance(action, np.ndarray):
+            raise ValueError("action must be ndarray")
+        if not isinstance(pscore, np.ndarray):
+            raise ValueError("pscore must be ndarray")
+
+        check_ope_inputs(
+            action_dist=action_dist,
+            position=position,
+            action=action,
+            reward=reward,
+            pscore=pscore,
+            estimated_rewards_by_reg_model=estimated_rewards_by_reg_model,
+        )
+        if position is None:
+            position = np.zeros(action_dist.shape[0], dtype=int)
+
         estimated_round_rewards = self._estimate_round_rewards(
             reward=reward,
             action=action,
@@ -848,10 +1000,10 @@ class SelfNormalizedDoublyRobust(DoublyRobust):
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> np.ndarray:
         """Estimate rewards for each round.
@@ -864,9 +1016,6 @@ class SelfNormalizedDoublyRobust(DoublyRobust):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
@@ -876,6 +1025,9 @@ class SelfNormalizedDoublyRobust(DoublyRobust):
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         estimated_rewards: array-like, shape (n_rounds,)
@@ -889,7 +1041,9 @@ class SelfNormalizedDoublyRobust(DoublyRobust):
         ]
         pi_e_at_position = action_dist[np.arange(n_rounds), :, position]
         estimated_rewards = np.average(
-            q_hat_at_position, weights=pi_e_at_position, axis=1,
+            q_hat_at_position,
+            weights=pi_e_at_position,
+            axis=1,
         )
         q_hat_factual = estimated_rewards_by_reg_model[
             np.arange(n_rounds), action, position
@@ -898,107 +1052,6 @@ class SelfNormalizedDoublyRobust(DoublyRobust):
         return estimated_rewards
 
 
-@dataclass
-class SwitchInverseProbabilityWeighting(DoublyRobust):
-    """Estimate the policy value by Switch Inverse Probability Weighting (Switch-IPW).
-
-    Note
-    -------
-    Switch-IPW aims to reduce the variance of the IPW estimator by using direct method
-    when the importance weight is large. This estimator estimates the policy value of a given evaluation policy :math:`\\pi_e` by
-
-    .. math::
-
-        & \\hat{V}_{\\mathrm{SwitchIPW}} (\\pi_e; \\mathcal{D}, \\tau) \\\\
-        & := \\mathbb{E}_{\\mathcal{D}} \\left[ \\sum_{a \\in \\mathcal{A}} \\hat{q} (x_t, a) \\pi_e (a|x_t) \\mathbb{I} \\{ w(x_t, a) > \\tau \\}
-         + w(x_t,a_t) r_t \\mathbb{I} \\{ w(x_t,a_t) \\le \\tau \\} \\right],
-
-    where :math:`\\mathcal{D}=\\{(x_t,a_t,r_t)\\}_{t=1}^{T}` is logged bandit feedback data with :math:`T` rounds collected by
-    a behavior policy :math:`\\pi_b`. :math:`w(x,a):=\\pi_e (a|x)/\\pi_b (a|x)` is the importance weight given :math:`x` and :math:`a`.
-    :math:`\\mathbb{E}_{\\mathcal{D}}[\\cdot]` is the empirical average over :math:`T` observations in :math:`\\mathcal{D}`.
-    :math:`\\tau (\\ge 0)` is a switching hyperparameter, which decides the threshold for the importance weight.
-    To estimate the mean reward function, please use `obp.ope.regression_model.RegressionModel`.
-
-    Parameters
-    ----------
-    tau: float, default=1
-        Switching hyperparameter. When importance weight is larger than this parameter, the DM estimator is applied, otherwise the IPW estimator is applied.
-        This hyperparameter should be larger than 1., otherwise it is meaningless.
-
-    estimator_name: str, default='switch-ipw'.
-        Name of off-policy estimator.
-
-    References
-    ----------
-    Miroslav Dudík, Dumitru Erhan, John Langford, and Lihong Li.
-    "Doubly Robust Policy Evaluation and Optimization.", 2014.
-
-    Yu-Xiang Wang, Alekh Agarwal, and Miroslav Dudík.
-    "Optimal and Adaptive Off-policy Evaluation in Contextual Bandits", 2016.
-
-    """
-
-    tau: float = 1
-    estimator_name: str = "switch-ipw"
-
-    def __post_init__(self) -> None:
-        """Initialize Class."""
-        assert (
-            self.tau >= 0.0
-        ), f"switching hyperparameter should be larger than 1, but {self.tau} is given"
-
-    def _estimate_round_rewards(
-        self,
-        reward: np.ndarray,
-        action: np.ndarray,
-        position: np.ndarray,
-        pscore: np.ndarray,
-        action_dist: np.ndarray,
-        estimated_rewards_by_reg_model: np.ndarray,
-        **kwargs,
-    ) -> float:
-        """Estimate rewards for each round.
-
-        Parameters
-        ----------
-        reward: array-like, shape (n_rounds,)
-            Reward observed in each round of the logged bandit feedback, i.e., :math:`r_t`.
-
-        action: array-like, shape (n_rounds,)
-            Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
-
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
-        pscore: array-like, shape (n_rounds,)
-            Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
-
-        action_dist: array-like, shape (n_rounds, n_actions, len_list)
-            Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
-
-        estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
-            Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
-
-        Returns
-        ----------
-        estimated_rewards: array-like, shape (n_rounds,)
-            Rewards estimated by the Switch-IPW estimator for each round.
-
-        """
-        n_rounds = action.shape[0]
-        iw = action_dist[np.arange(n_rounds), action, position] / pscore
-        switch_indicator = np.array(iw <= self.tau, dtype=int)
-        q_hat_at_position = estimated_rewards_by_reg_model[
-            np.arange(n_rounds), :, position
-        ]
-        pi_e_at_position = action_dist[np.arange(n_rounds), :, position]
-        estimated_rewards = (1 - switch_indicator) * np.average(
-            q_hat_at_position, weights=pi_e_at_position, axis=1,
-        )
-        estimated_rewards += switch_indicator * iw * reward
-        return estimated_rewards
-
-
 @dataclass
 class SwitchDoublyRobust(DoublyRobust):
     """Estimate the policy value by Switch Doubly Robust (Switch-DR).
@@ -1040,23 +1093,30 @@ class SwitchDoublyRobust(DoublyRobust):
 
     """
 
-    tau: float = 1
+    tau: float = 1.0
     estimator_name: str = "switch-dr"
 
     def __post_init__(self) -> None:
         """Initialize Class."""
-        assert (
-            self.tau >= 0.0
-        ), f"switching hyperparameter must be larger than or equal to zero, but {self.tau} is given"
+        if not isinstance(self.tau, (float, int)):
+            raise ValueError(
+                f"switching hyperparameter must be float or integer, but {self.tau} is given"
+            )
+        if self.tau != self.tau:
+            raise ValueError("switching hyperparameter must not be nan")
+        if self.tau < 0.0:
+            raise ValueError(
+                f"switching hyperparameter must be larger than or equal to zero, but {self.tau} is given"
+            )
 
     def _estimate_round_rewards(
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> float:
         """Estimate rewards for each round.
@@ -1069,9 +1129,6 @@ class SwitchDoublyRobust(DoublyRobust):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
@@ -1081,6 +1138,9 @@ class SwitchDoublyRobust(DoublyRobust):
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         estimated_rewards: array-like, shape (n_rounds,)
@@ -1098,7 +1158,9 @@ class SwitchDoublyRobust(DoublyRobust):
         ]
         pi_e_at_position = action_dist[np.arange(n_rounds), :, position]
         estimated_rewards = np.average(
-            q_hat_at_position, weights=pi_e_at_position, axis=1,
+            q_hat_at_position,
+            weights=pi_e_at_position,
+            axis=1,
         )
         estimated_rewards += switch_indicator * iw * (reward - q_hat_factual)
         return estimated_rewards
@@ -1161,18 +1223,25 @@ class DoublyRobustWithShrinkage(DoublyRobust):
 
     def __post_init__(self) -> None:
         """Initialize Class."""
-        assert (
-            self.lambda_ >= 0.0
-        ), f"shrinkage hyperparameter must be larger than or equal to zero, but {self.lambda_} is given"
+        if not isinstance(self.lambda_, (float, int)):
+            raise ValueError(
+                f"shrinkage hyperparameter must be float or integer, but {self.lambda_} is given"
+            )
+        if self.lambda_ != self.lambda_:
+            raise ValueError("shrinkage hyperparameter must not be nan")
+        if self.lambda_ < 0.0:
+            raise ValueError(
+                f"shrinkage hyperparameter must be larger than or equal to zero, but {self.lambda_} is given"
+            )
 
     def _estimate_round_rewards(
         self,
         reward: np.ndarray,
         action: np.ndarray,
-        position: np.ndarray,
         pscore: np.ndarray,
         action_dist: np.ndarray,
         estimated_rewards_by_reg_model: np.ndarray,
+        position: Optional[np.ndarray] = None,
         **kwargs,
     ) -> np.ndarray:
         """Estimate rewards for each round.
@@ -1185,9 +1254,6 @@ class DoublyRobustWithShrinkage(DoublyRobust):
         action: array-like, shape (n_rounds,)
             Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
-        position: array-like, shape (n_rounds,)
-            Positions of each round in the given logged bandit feedback.
-
         pscore: array-like, shape (n_rounds,)
             Action choice probabilities by a behavior policy (propensity scores), i.e., :math:`\\pi_b(a_t|x_t)`.
 
@@ -1197,6 +1263,9 @@ class DoublyRobustWithShrinkage(DoublyRobust):
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
             Expected rewards for each round, action, and position estimated by a regression model, i.e., :math:`\\hat{q}(x_t,a_t)`.
 
+        position: array-like, shape (n_rounds,), default=None
+            Positions of each round in the given logged bandit feedback.
+
         Returns
         ----------
         estimated_rewards: array-like, shape (n_rounds,)
@@ -1214,7 +1283,9 @@ class DoublyRobustWithShrinkage(DoublyRobust):
         ]
         pi_e_at_position = action_dist[np.arange(n_rounds), :, position]
         estimated_rewards = np.average(
-            q_hat_at_position, weights=pi_e_at_position, axis=1,
+            q_hat_at_position,
+            weights=pi_e_at_position,
+            axis=1,
         )
         estimated_rewards += shrinkage_weight * (reward - q_hat_factual)
         return estimated_rewards
