@@ -16,7 +16,6 @@ from .base import BaseContextFreePolicy
 
 # configurations to replicate the Bernoulli Thompson Sampling policy used in ZOZOTOWN production
 prior_bts_file = os.path.join(os.path.dirname(__file__), "conf", "prior_bts.yaml")
-# prior_bts_file = pkg_resources.resource_stream(__name__, "data/emperors.csv")
 with open(prior_bts_file, "rb") as f:
     production_prior_for_bts = yaml.safe_load(f)
 
@@ -49,13 +48,14 @@ class EpsilonGreedy(BaseContextFreePolicy):
     """
 
     epsilon: float = 1.0
-    policy_name: str = f"egreedy_{epsilon}"
 
     def __post_init__(self) -> None:
         """Initialize Class."""
-        assert (
-            0 <= self.epsilon <= 1
-        ), f"epsilon must be between 0 and 1, but {self.epsilon} is given"
+        if not 0 <= self.epsilon <= 1:
+            raise ValueError(
+                f"epsilon must be between 0 and 1, but {self.epsilon} is given"
+            )
+        self.policy_name = f"egreedy_{self.epsilon}"
         super().__post_init__()
 
     def select_action(self) -> np.ndarray:
@@ -126,7 +126,8 @@ class Random(EpsilonGreedy):
     policy_name: str = "random"
 
     def compute_batch_action_dist(
-        self, n_rounds: int = 1, n_sim: int = 100000,
+        self,
+        n_rounds: int = 1,
     ) -> np.ndarray:
         """Compute the distribution over actions by Monte Carlo simulation.
 
@@ -195,9 +196,10 @@ class BernoulliTS(BaseContextFreePolicy):
         """Initialize class."""
         super().__post_init__()
         if self.is_zozotown_prior:
-            assert (
-                self.campaign is not None
-            ), "`campaign` must be specified when `is_zozotown_prior` is True."
+            if self.campaign is None:
+                raise Exception(
+                    "`campaign` must be specified when `is_zozotown_prior` is True."
+                )
             self.alpha = production_prior_for_bts[self.campaign]["alpha"]
             self.beta = production_prior_for_bts[self.campaign]["beta"]
         else:
@@ -239,7 +241,9 @@ class BernoulliTS(BaseContextFreePolicy):
             self.reward_counts = np.copy(self.reward_counts_temp)
 
     def compute_batch_action_dist(
-        self, n_rounds: int = 1, n_sim: int = 100000,
+        self,
+        n_rounds: int = 1,
+        n_sim: int = 100000,
     ) -> np.ndarray:
         """Compute the distribution over actions by Monte Carlo simulation.
 
@@ -263,5 +267,8 @@ class BernoulliTS(BaseContextFreePolicy):
             selected_actions = self.select_action()
             for pos in np.arange(self.len_list):
                 action_count[selected_actions[pos], pos] += 1
-        action_dist = np.tile(action_count / n_sim, (n_rounds, 1, 1),)
+        action_dist = np.tile(
+            action_count / n_sim,
+            (n_rounds, 1, 1),
+        )
         return action_dist
