@@ -49,9 +49,10 @@ class LinEpsilonGreedy(BaseContextualPolicy):
 
     def __post_init__(self) -> None:
         """Initialize class."""
-        assert (
-            0 <= self.epsilon <= 1
-        ), f"epsilon must be between 0 and 1, but {self.epsilon} is given"
+        if not 0 <= self.epsilon <= 1:
+            raise ValueError(
+                f"epsilon must be between 0 and 1, but {self.epsilon} is given"
+            )
         self.policy_name = f"linear_epsilon_greedy_{self.epsilon}"
 
         super().__post_init__()
@@ -80,10 +81,15 @@ class LinEpsilonGreedy(BaseContextualPolicy):
             List of selected actions.
 
         """
+        if context.ndim != 2 or context.shape[0] != 1:
+            raise ValueError(
+                f"context shape must be (1, dim_context),but {context.shape} is given"
+            )
+
         if self.random_.rand() > self.epsilon:
             self.theta_hat = np.concatenate(
                 [
-                    self.A_inv[i] @ np.expand_dims(self.b[:, i], axis=1)
+                    self.A_inv[i] @ self.b[:, i][:, np.newaxis]
                     for i in np.arange(self.n_actions)
                 ],
                 axis=1,
@@ -165,9 +171,10 @@ class LinUCB(BaseContextualPolicy):
 
     def __post_init__(self) -> None:
         """Initialize class."""
-        assert (
-            0 <= self.epsilon <= 1
-        ), f"epsilon must be between 0 and 1, but {self.epsilon} is given"
+        if self.epsilon < 0:
+            raise ValueError(
+                f"epsilon must be positive scalar, but {self.epsilon} is given"
+            )
         self.policy_name = f"linear_ucb_{self.epsilon}"
 
         super().__post_init__()
@@ -196,9 +203,13 @@ class LinUCB(BaseContextualPolicy):
             List of selected actions.
 
         """
+        if context.ndim != 2 or context.shape[0] != 1:
+            raise ValueError(
+                f"context shape must be (1, dim_context),but {context.shape} is given"
+            )
         self.theta_hat = np.concatenate(
             [
-                self.A_inv[i] @ np.expand_dims(self.b[:, i], axis=1)
+                self.A_inv[i] @ self.b[:, i][:, np.newaxis]
                 for i in np.arange(self.n_actions)
             ],
             axis=1,
@@ -264,9 +275,6 @@ class LinTS(BaseContextualPolicy):
     batch_size: int, default=1
         Number of samples used in a batch parameter update.
 
-    alpha_: float, default=1.
-        Prior parameter for the online logistic regression.
-
     random_state: int, default=None
         Controls the random seed in sampling actions.
 
@@ -274,7 +282,7 @@ class LinTS(BaseContextualPolicy):
 
     def __post_init__(self) -> None:
         """Initialize class."""
-        self.policy_name = f"linear_ts"
+        self.policy_name = "linear_ts"
 
         super().__post_init__()
         self.A_inv = np.concatenate(
@@ -303,17 +311,16 @@ class LinTS(BaseContextualPolicy):
         """
         theta_hat = np.concatenate(
             [
-                self.A_inv[i] @ np.expand_dims(self.b[:, i], axis=1)
+                self.A_inv[i] @ self.b[:, i][:, np.newaxis]
                 for i in np.arange(self.n_actions)
             ],
             axis=1,
         )
         theta_sampled = np.concatenate(
             [
-                np.expand_dims(
-                    self.random_.multivariate_normal(theta_hat[:, i], self.A_inv[i]),
-                    axis=1,
-                )
+                self.random_.multivariate_normal(theta_hat[:, i], self.A_inv[i])[
+                    :, np.newaxis
+                ]
                 for i in np.arange(self.n_actions)
             ],
             axis=1,
@@ -353,4 +360,3 @@ class LinTS(BaseContextualPolicy):
                 np.copy(self.A_inv_temp),
                 np.copy(self.b_temp),
             )
-
